#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>

#include "rpm-internal.h"

static int vinit = 0;

static void getDimensions(Tcl_Interp* interp, int level,
			  rpmBinary* vp, char* list)
{
    int numnewargs;
    char** newargs;

    Tcl_SplitList(interp, list, &numnewargs, &newargs);
    if(numnewargs == 1) {
	free((char *) newargs);
	if(level == 0) {
	    vp->dimens = 1;
	    vp->dim = (int *) malloc(vp->dimens * sizeof(int));
	    vp->dim[level] = 1;
	    return;
	}
	vp->dimens = level;
	vp->dim = (int *) malloc(vp->dimens * sizeof(int));
	return;
    }
    getDimensions(interp, level + 1, vp, newargs[0]);
    vp->dim[level] = numnewargs;
    free((char *) newargs);
}

static int fillValues(Tcl_Interp* interp, int level, rpmBinary* m,
		      int offs, int nargs, char** args)
{
    char** newargs;
    int numnewargs;
    int newoffs;
    int i;

    if(vinit)
	fprintf(stderr, "level %d  offset %d  args %d\n",
		m->dimens, offs, nargs);
    if(level < m->dimens) {
	for(i = 0; i < nargs; i++) {
	    if(Tcl_SplitList(interp, args[i], &numnewargs, &newargs)
	       != TCL_OK) {
		Tcl_AppendResult(interp, "bad matrix initializer list form: ",
				 args[i], (char *) NULL);
		return TCL_ERROR;
	    }
	    if(level > 0)
		newoffs = offs * m->dim[level - 1] + i;
	    else
		newoffs = 0;
	    fillValues(interp, level + 1, m, newoffs, numnewargs, newargs);
	    free((char *) newargs);
	}
	return TCL_OK;
    }
    
    for(i = 0; i < nargs; i++) {
	newoffs = offs * m->dim[level - 1] + i;
	m->v[newoffs] = atof(args[i]);
	if(vinit)
	    fprintf(stderr, "\ta[%d] = %s\n",  newoffs, args[i]);
    }
    return TCL_OK;
}

rpmBinary rpmAtof(Tcl_Interp* interp, char* list)
{
    rpmBinary v;
    int l, i;

    getDimensions(interp, 0, &v, list);
    l = 1;
    for(i = 0; i < v.dimens; i++)
	l *= v.dim[i];
    v.space = l * sizeof(rpmFloat);
    v.v = (rpmFloat *) malloc(v.space);
    v.dynamic = 1;
    v.usedas = rNumbers;
    if(fillValues(interp, 0, &v, 0, 1, &list) != TCL_OK) {
	Tcl_AppendResult(interp, "rpmAtof: fillValues failed.\n", 0);
	v.good = 0;
	free(v.v);
	free(v.dim);
	v.dimens = 0;
	v.space = 0;
	v.dynamic = 0;
    } else
	v.good = 1;
    return v;
}
