#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"

static char rcsid[] = "$Id: rpmGenerate.c,v 1.5 1999/08/09 13:51:11 martin Exp $";

/*
**	to from nsteps sequence
*/
int rpmSequenceSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int k;
    double xmin, xmax, dx;
    double x;
    int i, npts;
    rpmBinary r;

    npts = rpmPopScalar();
    xmin = rpmPopScalar();
    xmax = rpmPopScalar();
    if(npts > 1)
	dx = (xmax - xmin)/(npts - 1.0);
    else
	dx = 0.0;
    r = rpmNewBinaryD1(npts);
    k = 0;
    x = xmin;
    for(i = 0; i < npts; i++) {
	r.v[i] = x;
	x += dx;
    }
    rpmPushB(r);
    return TCL_OK;
}

/*
**	count urandom
*/
int rpmUrandomSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int i, npts;
    rpmBinary r;
    double drand48(void);

    npts = rpmPopScalar();
    r = rpmNewBinaryD1(npts);
    for(i = 0; i < npts; i++) {
       r.v[i] = drand48();
    }
    rpmPushB(r);
    return TCL_OK;
}

/*
**	list count replicate
*/
int rpmReplicateSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    int i, j, llen, rep;
    rpmBinary list;
    rpmBinary r;
    int* dlist;
    int lx, la, lr;

    rep = rpmPopScalar();
    list = rpmPop();
    llen = rpmLength(list);

    dlist = (int *) malloc(list.dimens * sizeof(int));
    for(i = 0; i < list.dimens; i++)
      dlist[i] = list.dim[i];
    dlist[list.dimens - 1] *= rep;
    r = rpmNewBinary(list.dimens, dlist);
    lx = 1;
    for(i = 0; i < (r.dimens - 1); i++)
      lx *= r.dim[i];
    la = list.dim[list.dimens - 1];
    lr = r.dim[r.dimens - 1];
    for(i = 0; i < lx; i++) {
      for(j = 0; j < rep; j++)
	memmove(r.v + i * lr + j * la,
		list.v + i * la, la * sizeof(r.v[0]));
    }
    rpmFreeBinary(list);
    rpmPushB(r);
    free(dlist);
    return TCL_OK;
}
/*
**	interpolate elements into a general rectilinear	array:
**		if(step)	use flat interpolation
**		else		use linear interpolation
*/
static void interpolate_matrix(rpmFloat* orig, rpmFloat* new, int nd,
			       int od[], int mpy[], int level, int step)
{
    int        	i;
    long       	j;
    long       	l;
    long       	ostep;
    long       	nstep;
    rpmFloat* 	ir;
    rpmFloat  	z1, z2;
    rpmFloat* 	farg;

    if(level >= nd)
	return;

    ostep = 1;
    nstep = 1;
    for(i = level + 1; i < nd; i++) {
	ostep *= od[i];
	nstep *= (od[i] * mpy[i] - mpy[i] + 1);
    }

    if(level == (nd - 1)) {
	for(i = 0; i < od[level]; i++) {
	    for(j = 0; j < mpy[level]; j++) {
		l = i * mpy[level] + j;
		if(step) {
		    if((j == 0) || (j < (mpy[level]/2)))
			new[l] = orig[i];
		    else
			new[l] = orig[i + 1];
		} else {
		    if(mpy[level] > 1)
			z1 = ((float) j) / mpy[level];
		    else
			z1 = 0;
		    z2 = 1.0 - z1;
		    if(j > 0)
			new[l] = z2 * orig[i] + z1 * orig[i + 1];
		    else
			new[l] = orig[i];
		}
		if(i == (od[level] - 1))
		    break;
	    }
	}
	return;
    }
    ir = (rpmFloat *) malloc(sizeof(rpmFloat) * ostep);
    for(i = 0; i < od[level]; i++) {
	for(j = 0; j < mpy[level]; j++) {
	    if(mpy[level] > 1)
		z1 = ((float) j) / mpy[level];
	    else
		z1 = 0;
	    z2 = 1.0 - z1;
	    if(j > 0) {
		for(l = 0; l < ostep; l++)
		    if(step) {
			if(j < (mpy[level]/2))
			    ir[l] = orig[l + ostep * i];
			else
			    ir[l] = orig[l + ostep * i + ostep];
		    } else {
			ir[l] = z2 * orig[l + ostep * i]
			    + z1 * orig[l + ostep * i + ostep];
		    }
		farg = &ir[0];
	    } else
		farg = orig + ostep * i;
	    interpolate_matrix(farg, new + nstep * (j + i * mpy[level]),
			       nd, od, mpy, level + 1, step);
	    if(i == (od[level] - 1))
		break;
	}
    }
    free(ir);
}

int realExpand(ClientData dummy, Tcl_Interp* interp, int step)
{
    rpmBinary	v;
    rpmBinary	el;
    int         nd;
    int* 	xlist;
    int		newels;
    int         i;
    int*	newd;
    rpmBinary	r;

    el = rpmPop();
    v = rpmPop();

    nd = rpmLength(el);
    if(nd < 1) {
	Tcl_AppendResult(interp, "expand (sexpand): expansion-list too short",
			 0);
	return TCL_ERROR;
    }
    if(nd < v.dimens)
	nd = v.dimens;
    xlist = (int *) malloc(nd * sizeof(int));
    newd  = (int *) malloc(nd * sizeof(int));
    newels = 1;
    for(i = 0; i < rpmLength(el); i++) {
	xlist[i] = el.v[i];
	newd[i]  = (xlist[i] * v.dim[i] - xlist[i] + 1);
	newels  *= newd[i];
    }
    for(; i < nd; i++) {
	xlist[i] = 1;
	newd[i] = v.dim[i];
	newels *= newd[i];
    }

    r = rpmNewBinary(v.dimens, newd);
    free((char *) newd);

    interpolate_matrix(v.v, r.v, r.dimens, v.dim, xlist, 0, step);
    rpmFreeBinary(v);
    rpmFreeBinary(el);
    free((char *) xlist);
    rpmPushB(r);
    return TCL_OK;
}

/*
**     list expansion-list expand
*/
int rpmExpandSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return realExpand(dummy, interp, 0);
}

/*
**     list expansion-list sexpand
*/
int rpmSExpandSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return realExpand(dummy, interp, 1);
}

static void shrink_matrix(rpmFloat* orig, rpmFloat* new, int nd, int od[],
			     int div[], int level)
{
    int       i;
    long      ostep;
    long      nstep;
    long      nn;

    if(level >= nd)
	return;

    ostep = 1;
    nstep = 1;
    for(i = level + 1; i < nd; i++) {
	ostep *= od[i];
	nstep *= (int) ceil(((double) od[i]) / div[i]);
    }

    nn = (int) ceil(((double) od[level]) / div[level]);

    if(level == (nd - 1)) {
	for(i = 0; i < nn; i++)
	    new[i] = orig[i * div[level]];
	return;
    }

    for(i = 0; i < nn; i++)
	shrink_matrix(orig + ostep * i * div[level],
			 new + nstep * i,
			 nd, od, div, level + 1);
}
/*
**    list shrink-list shrink
*/
int rpmShrinkSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary	v;
    rpmBinary	el;
    int         nd;
    int		i;
    int	        *xlist;
    int	        newels;
    int*	newd;
    rpmBinary	r;

    el = rpmPop();
    v  = rpmPop();

    nd = rpmLength(el);
    if(nd < 1) {
	Tcl_AppendResult(interp, "shrink: expansion list too short", 0);
	return TCL_ERROR;
    }
    if(nd < v.dimens)
	nd = v.dimens;
    xlist = (int *) malloc(nd * sizeof(int));
    newd  = (int *) malloc(nd * sizeof(int));
    newels = 1;
    for(i = 0; i < rpmLength(el); i++) {
	xlist[i] =  el.v[i];
	newd[i]  =  (int) ceil(((double) v.dim[i]) / xlist[i]);
	newels   *= newd[i];
    }
    for(; i < nd; i++) {
	xlist[i] =  1;
	newd[i]  =  v.dim[i];
	newels   *= newd[i];
    }

    r = rpmNewBinary(v.dimens, newd);
    free((char *) newd);

    shrink_matrix(v.v, r.v, r.dimens, v.dim, xlist, 0);
    free((char *) xlist);
    rpmFreeBinary(v);
    rpmFreeBinary(el);
    rpmPushB(r);
    return TCL_OK;
}


