#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"

static char rcsid[] = "$Id: rpmList.c,v 1.6 1999/08/09 13:51:11 martin Exp $";

/*
**	list dimensions
*/
int rpmDimensionsSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, retv;
    int i;

    rlist = rpmPop();
    retv = rpmNewBinaryD1(rlist.dimens);
    for(i = 0; i < rlist.dimens; i++)
      retv.v[i] = rlist.dim[i];
    rpmFreeBinary(rlist);
    rpmPushB(retv);
    return TCL_OK;
}
/*
**  return the offset and size of the submatrix pointed to by the
**  index list il (C style) in a matrix with dimensions dl.
**
**  results are returned to int* offset and int* bsize.
*/
static void moffset(int numd, int* dl, int numi, rpmFloat* il,
		   int* offset, int* bsize)
{
  int i;
  int lo, bo;

/*
**  bo is the size of the block il points to
*/
  bo = 1;
  for(i = numi; i < numd; i++)
    bo *= dl[i];
/*
**  lo is the offset of the block in units of bo.
*/
  lo = il[0];
  for(i = 1; i < numi; i++)
    lo = lo * dl[i] + il[i];

  *bsize = bo;
  *offset = lo * bo;
  return;
}

/*
**	list index-list get
*/
int rpmGetSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, ilist, retv;
    int offset, size;

    ilist = rpmPop();
    rlist = rpmPop();
    moffset(rlist.dimens, rlist.dim, rpmLength(ilist), ilist.v,
	    &offset, &size);

    if(rlist.dimens == rpmLength(ilist))
	retv = rpmNewBinaryD1(1);
    else
	retv = rpmNewBinary(rlist.dimens - rpmLength(ilist),
			    rlist.dim + rpmLength(ilist));
    memcpy(retv.v, rlist.v + offset, size * sizeof(retv.v[0]));
    rpmFreeBinary(rlist);
    rpmFreeBinary(ilist);
    rpmPushB(retv);
    return TCL_OK;
}
/*
**	list v-list ilist put
*/
int rpmPutSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary rlist, vlist, ilist;
    int offset, size;

    ilist = rpmPop();
    vlist = rpmPop();
    rlist = rpmPop();
    moffset(rlist.dimens, rlist.dim, rpmLength(ilist), ilist.v,
	    &offset, &size);
    if(rlist.dynamic == 0)
	rlist = rpmDeepCopy(rlist);
    size = rpmLength(vlist);
    if((offset + size) > rpmLength(rlist))
	size = rpmLength(rlist) - offset;
    memcpy(rlist.v + offset, vlist.v, size * sizeof(vlist.v[0]));
    rpmFreeBinary(vlist);
    rpmFreeBinary(ilist);
    rpmPushB(rlist);
    return TCL_OK;
}

