#include <stdlib.h>
#include <tcl.h>
#include <stdio.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"

static char rcsid[] = "$Id: rpmObjStack.c,v 1.4 1999/08/09 13:51:11 martin Exp $";

rpmBinary rpmBadB = {0, 0, 0, 0, 0};

/*
**	stack management
*/
typedef struct {
    rpmBinary* 	stack;
    int		Size;
    int		TOS;
    void*	previous;
} rpmStack;

static int 	stackChunk = 32;
static 		rpmStack* cs = 0;
static int 	stackcount = 0;

rpmBinary rpmNewBinary(int dims, int* dimlist)
{
    rpmBinary v;
    int i;

    v.good = 1;
    v.vtype = rFloatT;
    v.usedas = rNumbers;
    v.dynamic = 1;
    v.dimens = dims;
    if(v.dimens > 0) {
	v.dim = (int *) malloc(v.dimens * sizeof(int));
	memcpy(v.dim, dimlist, v.dimens * sizeof(int));
    } else
	v.dim = 0;
    if(dims <= 0)
	v.space = 0;
    else {
	v.space = dimlist[0] * sizeof(rpmFloat);
	for(i = 1; i < dims; i++)
	    v.space *= dimlist[i];
    }
    v.hname = 0;
    if(v.space == 0) {
	v.v = 0;
    } else {
	v.v = (rpmFloat *) malloc(v.space);
    }
    return v;
}

rpmBinary rpmNewBinaryLike(rpmBinary r)
{
    return rpmNewBinary(r.dimens, r.dim);
}

rpmBinary rpmNewBinaryD1(int n)
{
    return rpmNewBinary(1, &n);
}

rpmBinary rpmNewBinaryD2(int m, int n)
{
    int l[2];
    
    l[0] = m;
    l[1] = n;
    return rpmNewBinary(2, &l);
}

void rpmFreeBinary(rpmBinary v)
{
    switch(v.usedas) {
    case rHandle:
	break;
    case rString:
	if(v.string != 0)
	    free(v.string);
	break;
    case rNumbers:
	if(v.dynamic) {
	    if(v.v != 0)
		free(v.v);
	    if(v.dim != 0)
		free(v.dim);
	}
	break;
    default:
	break;
    }
}

rpmBinary rpmDeepCopy(rpmBinary r)
{
    rpmBinary q;

    if(r.good == 0)
	return r;
    switch(r.usedas) {
    case rCommand:
    case rHandle:
	q = r;
	break;
    case rString:
	q = r;
	q.string = malloc(strlen(r.string) + 1);
	strcpy(q.string, r.string);
	break;
    case rNumbers:
	q = rpmNewBinary(r.dimens, r.dim);
	q.good = r.good;
	memcpy(q.v, r.v, r.space);
	q.hname = r.hname;
	break;
    }
    return q;
}

int rpmLength(rpmBinary r)
{
    int n, i;
    if(r.dimens < 1)
	return 0;
    n = r.dim[0];
    for(i = 1; i < r.dimens; i++)
	n *= r.dim[i];
    return n;
}

void rpmNewStack()
{
    rpmStack* s;
    s = (rpmStack *) malloc(sizeof(rpmStack));
    s->Size = stackChunk;
    s->stack = (rpmBinary *) malloc(s->Size * sizeof(rpmBinary));
    s->TOS = 0;
    s->previous = cs;
    cs = s;
    ++stackcount;
}

void rpmDeleteStack()
{
    rpmStack* s;

    while(cs->TOS > 0)
	rpmFreeBinary(cs->stack[--cs->TOS]);
    --stackcount;
    s = cs;
    cs = s->previous;
    free(s->stack);
    free(s);
}

int rpmStackCount()
{
    return stackcount;
}

void rpmPushB(rpmBinary vb)
{
    rpmBinary* ns;

    if(cs->TOS >= cs->Size) {
	cs->Size += stackChunk;
	ns = (rpmBinary*) malloc(cs->Size * sizeof(rpmBinary));
	memcpy(ns, cs->stack, cs->TOS * sizeof(rpmBinary));
	if(cs->stack != 0)
	    free(cs->stack);
	cs->stack = ns;
    }
    cs->stack[cs->TOS++] = vb;
}

rpmBinary rpmPop()
{
    if(cs->TOS == 0)
	return rpmBadB;
    return cs->stack[--cs->TOS];
}

double rpmPopScalar()
{
    double d;
    rpmBinary r;

    if(cs->TOS == 0)
	return 0.0;
    r =  cs->stack[--cs->TOS];
    d = r.v[0];
    rpmFreeBinary(r);
    return d;
}

int rpmStackDepth()
{
    return cs->TOS;
}
