#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"

static char rcsid[] = "$Id: rpmShape.c,v 1.5 1999/08/09 13:51:11 martin Exp $";

/*
**	cat
*/
int rpmCatSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary a, b, c;
    int i;
    int* dlist;
    int lx;
    int la, lb, lc;

    b = rpmPop();
    a = rpmPop();
    if(b.dimens != a.dimens) {
      Tcl_AppendResult(interp, "rpm cat: args of different dims", 0);
      return TCL_ERROR;
    }
    for(i = 0; i < (b.dimens - 1); i++)
      if(a.dim[i] != b.dim[i]) {
	Tcl_AppendResult(interp, "rpm cat: args of different shape", 0);
	return TCL_ERROR;
      }
    dlist = (int *) malloc(a.dimens * sizeof(int));
    for(i = 0; i < a.dimens; i++)
      dlist[i] = a.dim[i];
    dlist[a.dimens - 1] += b.dim[b.dimens - 1];
    c = rpmNewBinary(a.dimens, dlist);
    lx = 1;
    for(i = 0; i < (c.dimens - 1); i++)
      lx *= c.dim[i];
    la = a.dim[a.dimens - 1];
    lb = b.dim[b.dimens - 1];
    lc = c.dim[c.dimens - 1];
    for(i = 0; i < lx; i++) {
      memmove(c.v + i * lc,        a.v + i * la, la * sizeof(c.v[0]));
      memmove(c.v + i * lc + la, b.v + i * lb, lb * sizeof(c.v[0]));
    }
    rpmFreeBinary(a);
    rpmFreeBinary(b);
    free(dlist);
    rpmPushB(c);
    return TCL_OK;
}
/*
**	v1 flatten
*/
int rpmFlattenSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary a, c;
    int l;

    a = rpmPop();
    l = rpmLength(a);
    c = rpmNewBinaryD1(l);
    memmove(c.v, a.v, l * sizeof(c.v[0]));
    rpmFreeBinary(a);
    rpmPushB(c);
    return TCL_OK;
}
/*
**	v1 i-list reshape
*/
int rpmReshapeSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary a, b, c;
    int* dlist;
    int i;
    int lc, la, lcpy;

    b = rpmPop();
    a = rpmPop();
    if(b.dimens != 1) {
      Tcl_AppendResult(interp, "rpm reshape: ilist a matrix?", 0);
      return TCL_ERROR;
    }
    dlist = (int *) malloc(b.dim[0] * sizeof(int));
    for(i = 0; i <b.dim[0]; i++)
      dlist[i] = (int) (b.v[i] + 0.001);
    c = rpmNewBinary(b.dim[0], dlist);
    lc = rpmLength(c);
    la = rpmLength(a);
    lcpy = (la > lc) ? lc : la;
    memmove(c.v, a.v, lcpy * sizeof(c.v[0]));
    for(i = lcpy; i < lc; i++)
      c.v[i] = 0.0;
    rpmFreeBinary(a);
    rpmFreeBinary(b);
    free(dlist);
    rpmPushB(c);
    return TCL_OK;
}
/*
**	transpose
*/
int rpmTransposeSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary a, b;
    int i, j, na, nb;

    a = rpmPop();
    if(a.dimens > 2) {
      Tcl_AppendResult(interp, "rpm transpose: too many dimensions", 0);
      return TCL_ERROR;
    }
    if(a.dimens == 2) {
	b = rpmNewBinaryLike(a);
	i = b.dim[0];
	b.dim[0] = b.dim[1];
	b.dim[1] = i;
	for(j = 0; j < a.dim[0]; j++)
	    for(i = 0; i < a.dim[1]; i++) {
		na = i + j * a.dim[1];
		nb = j + i * b.dim[1];
		b.v[nb] = a.v[na];
	    }
	rpmFreeBinary(a);
	a = b;
    }
    rpmPushB(a);
    return TCL_OK;
}


/*
**	flip
*/
int rpmFlipSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary a;
    int i, n;
    double y;

    a = rpmPop();
    n = rpmLength(a);
    for(i = 0;i < n/2; i++) {
	y = a.v[i];
	a.v[i] = a.v[n - 1 - i];
	a.v[n - 1 - i] = y;
    }
    rpmPushB(a);
    return TCL_OK;
}


